
-- Gravity/Coroutine example
--
-- This script demonstrates coroutines in Lua. A number of particles are created
-- and influenced by "gravity". 
--
-- Copyright (c) 2001-2004 Nick Trout
-- This file is part of Doris. (http://doris.sf.net/)
-- Doris is free software. See terms of GNU license in docs.
-- $Id: gravity.lua,v 1.7 2004/02/28 22:26:45 trout Exp $


local rotatemat = Matrix:new()  -- create a matrix to rotate the scene
local gparts = {}  -- this will hold positions of all particles
local dt = 1/30  -- time step
local G = 1
local fs = .45 -- force scale
local createthresh = 0.96 -- particle spawn rate
local showvel, showforce, randcreate = 0, 0, 0

-- We create a coroutine from this function to initialise and update the
-- particle. Coroutines can yield back to the function which called (resumed) 
-- them without losing their local state. The particle is updated in a loop
-- which yields back to the render update loop every frame.
-- Params: The coroutine created and the particle position.
--
function partCoRoutine(coself, px, py, pz)
    -- initialise local variables (that persist until coroutine dead/exits)
    local life = math.random(500)+500
    local x, y, z = px, py, pz
    local vx, vy, vz = 0, 0, 0  -- velocity
    local sqrt = math.sqrt  -- optimisation
    
    coroutine.yield()  -- suspend coroutine after initialisation
    
    -- loop until the particles life expires
    while life>0 do
        life = life-1  -- countdown
        
        glPushMatrix()  
        glTranslate(x, y, z)  -- translate our drawing origin to the particle
            
        -- Work out out the change in velocity due to forces from the
        -- other particles.
        -- Newtons: F=m.a, m=1, -> F=a
        -- Gravity: F=sum(-G.m1.m2/d^2), m=1 -> F=a=sum(-G/d^2)
        -- NB: but 1/d^2 creates very large forces at close range so use 1/d
        -- dv = a.dt = sum(-G/d).dt
        for pco, ppos in gparts do
            if pco~=coself then
                local dx, dy, dz = ppos[1]-x, ppos[2]-y, ppos[3]-z
                local dist2 = dx*dx + dy*dy + dz*dz
                local dist = sqrt(dist2)
                local norm = 1/dist  -- normalised distance to make force unit vector
                local dv = G/dist*dt -- dv=a*dt
                
                -- show some optional force lines between the particles
                if showforce>0 then
                    glBegin(GL_LINES)
                        local c = dv*70
                        glColor(c, 0, 1-c)
                        glVertex(0, 0, 0)
                        glVertex(dx*fs, dy*fs, dz*fs)
                    glEnd()
                end
                
                vx, vy, vz = vx + dx*norm*dv, vy + dy*norm*dv, vz + dz*norm*dv
            end
        end
        
        -- update the position from the new velocity
        x, y, z = x + vx*dt, y + vy*dt, z + vz*dt
        
        -- draw the particle
        glColor(.6, .6, .6)
        glutSolidCube(.2)
        
        if showvel>0 then
            glBegin(GL_LINES)
                glColor(0.8, 0.5, 0)
                glVertex(0, 0, 0)
                glVertex(vx, vy, vz)
            glEnd()
        end
        glPopMatrix()
        
        coroutine.yield(x, y, z)  -- suspend coroutine & return updated pos
    end
    -- upon exiting this function the coroutines state will become "dead"
end

-- This function creates a single new particle. We create a coroutine
-- using the particle update function above.
--
function createParticle()   
    -- create the coroutine to update the particle
    local co = coroutine.create(partCoRoutine)
    
    -- Give the particle a position (and publish it in the global table so
    -- other particles know it exists and can calculate correct gravity)
    local x ,y, z = math.random(10)-5, math.random(10)-5, math.random(10)-5
    gparts[co] = {x, y, z}
    
    -- This first resume is an initialisation call.
    coroutine.resume(co, co, x, y, z)
end

-- The callback function which renders our scene.
--
function renderScene()
    -- Clear the screen and depth buffer
    glClear(GL_DEPTH_BUFFER_BIT + GL_COLOR_BUFFER_BIT)

    -- Tell the camera/eye what to look at
    gluLookAt(0,0,20, 0,0,0, 0,1,0)
    glMatrixMode(GL_MODELVIEW)

    -- Draw the scene.
    glPushMatrix()
        glLoadMatrix(rotatemat)

        local newparts = {}        
        for pco, pos in gparts do
            local ret, x, y, z = coroutine.resume(pco, pco)
            assert(ret, x)
            --print (pco, x, y, z)
            if coroutine.status(pco)~="dead" then
                newparts[pco] = {x, y, z}
            end
        end
        gparts = newparts
    glPopMatrix()
    
    if randcreate>0 and math.random()>createthresh then
        createParticle()
    end
end

-- Create our window and specify a name and window rendering function
--
wmain = Window:create {
    title = "Doris: Gravity / Coroutines example",
    render = renderScene, -- specify the redraw function for this window
    idle = function() return 1 end -- keep forcing redraw
}

-- Put a sub window at the right of the screen
subwin = SubWindow { parent = wmain, side = "right" }

Rotator {
    parent = subwin,       -- parent of this widget
    text = "Rotate scene", -- text label for this widget
    value = rotatemat,     -- matrix that rotator manipulates
    line = "-",            -- vertical spacing line drawn
    spin = 0.99     -- allow the rotator to spin with little damping when released
}

Button {
    parent = subwin,
    text = "Make particle",
    update = createParticle,
}

Button {
    parent = subwin,
    text = "Kill particles",
    update = function() gparts={} end,
}

Checkbox{
    parent = subwin,
    text = "Show velocity",
    value = showvel,
    update = function(x) showvel = x end
    }

Checkbox{
    parent = subwin,
    text = "Show forces",
    value = showforce,
    update = function(x) showforce = x end
    }

Checkbox{
    parent = subwin,
    text = "Random create",
    value = randcreate,
    update = function(x) randcreate = x end
    }

    