
-- Texture generation
--
-- Copyright (c) 2001-2004 Nick Trout
-- This file is part of Doris. (http://doris.sf.net/)
-- Doris is free software. See terms of GNU license in docs.
-- $Id: texgen.lua,v 1.5 2004/02/25 02:42:51 trout Exp $

rotatemat = Matrix:new() -- create scene rotation matrix

texture = {
    size = 64,
    fns = { 

["checks"] = 
[[
  local v = math.mod(math.floor(t.x/8) + math.floor(t.y/8),2)
  return v,v,v
]],

["radial gradient"] =
[[
  local dx,dy = t.centre-t.x,t.centre-t.y
  local d = 1 - math.sqrt(dx*dx+dy*dy)/t.centre
  if d<0 then d=0 end
  return d,d,0
]],

["tartan"] =
[[
  local r = math.sin(3*360*t.x/t.size)*.5+.5
  local g = math.sin(3*360*t.y/t.size)*.5+.5
  return r,g,0
]],

["blob grid"] = 
[[
  local h = math.sin(5.5*360*t.x/t.size)*.5+.5
  local v = math.sin(5.5*360*t.y/t.size)*.5+.5
  local i = (h+v)*.5
  return i,i,i
]],

["radial RGB"] = 
[[
  local a = math.atan2(t.y-t.centre,t.x-t.centre)
  if a<0 then a=a+360 end
  local c = {0,0,0}
  local c1 = math.floor(a/120)
  local c2 = math.mod(c1+1,3)
  local v = math.mod(a,120)/120
  c[c1+1],c[c2+1] = 1-v,v
  return c[1],c[2],c[3]
]],

["repeat binary"] =
[[
  local nb = t.y 
  local m = 2^math.mod(t.x,8)
  local b = math.mod(nb/m,2)
  return b,b,b
]],

["fractalish"] = 
[[
  local a = math.atan2(t.y-t.centre,t.x-t.centre)
  if a<0 then a=a+360 end
  local c = {0,0,0}
  local c1 = math.floor(a/120)
  local c2 = math.mod(c1+1,3)
  local v = math.mod(a,120)/120
  c[c1+1],c[c2+1] = 1-v,v
  local dx, dy = t.centre-t.x, t.centre-t.y
  local d = math.sqrt(dx*dx+dy*dy)/t.centre
  if d<0 then d=0 end
  local r = 1-d+c[1]
  local g = 1-d+c[2]
  local b = 1-d+c[3]
  return r,g,b
]],

["colour spectrum"] = 
[[
  local a = math.atan2(t.y-t.centre, t.x-t.centre)
  if a<0 then a=a+360 end
  local c = {0,0,0}
  local c1 = math.floor(a/120)
  local c2 = math.mod(c1+1, 3)
  local v = math.mod(a, 120)/120
  c[c1+1], c[c2+1] = 1-v, v
  local dx, dy = t.centre-t.x, t.centre-t.y
  local d = math.sqrt(dx*dx + dy*dy) / t.centre
  if d<0 then d=0 end
  if d>1 then return 0,0,0 end
  local r = 1-d+c[1]
  local g = 1-d+c[2]
  local b = 1-d+c[3]
  if r>1 then r=1 end
  if g>1 then g=1 end
  if b>1 then b=1 end
  return r,g,b
]]
    }
}

defaultfn = texture.fns.checks

function createObjects()
  assert(texture.id, "texture not created")
  
  CallList:create("sphere")
  glBindTexture(GL_TEXTURE_2D,texture.id)
  Quadric:sphere{texture=1, solid=1, slices=20, stacks=20 }
  CallList:endList()
  
  CallList:create("poly")
  glBindTexture(GL_TEXTURE_2D,texture.id)
  glBegin(GL_QUADS)
    glTexCoord(0.0, 0.0); glVertex(-1.0, -1.0, 0);
    glTexCoord(1.0, 0.0); glVertex( 1.0, -1.0,  0);
    glTexCoord(1.0, 1.0); glVertex( 1.0,  1.0,  0);
    glTexCoord(0.0, 1.0); glVertex(-1.0,  1.0,  0);
  glEnd()
  CallList:endList()
end

function createTexture()
  glEnable(GL_TEXTURE_2D)
  
  -- create texture buffer
  local size = texture.size
  print("Creating texture of size "..size.." x "..size)
  texture.pixels = gllCreateBuffer(size*size*3,GL_BYTE)
  assert(texture.pixels)
  
  local texs = {0}
  glGenTextures(1,texs)
  texture.id = texs[1]
  
  -- create GL texture
  glBindTexture(GL_TEXTURE_2D,texture.id)
  glTexParameter(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR)
  glTexParameter(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR)
  _glErrorCheck("after texture init")
end

function applyFunction()
  assert(texture.fn,"no function")
  local pixels = texture.pixels
  local size = texture.size
  local fn = texture.fn
  local params = {
    size = size,
    centre = size/2,
  }
  local p = 0
  for y=0,size-1 do
    for x=0,size-1 do
      params.x,params.y = x,y
      local r,g,b = fn(params)
      gllSetBuffer(pixels, p, r*255)
      gllSetBuffer(pixels, p+1, g*255)
      gllSetBuffer(pixels, p+2, b*255)
      p = p+3
    end
  end
  glBindTexture(GL_TEXTURE_2D,texture.id)
  glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, size,size, 0,
            GL_RGB, GL_UNSIGNED_BYTE, texture.pixels)
  _glErrorCheck("after texture generation")
end

function setFunction(fnsource)
  fnsource = "texture.fn = function(t)"..fnsource.."end"
  print("\n\nFunction:\n")
  print(fnsource)
  local fncomp = loadstring(fnsource)
  if not fncomp then
    texture.fn = function() return 1,0,0 end
  end
  fncomp()
  assert(type(texture.fn)=="function")
  applyFunction()
end

-- The callback function which renders our scene.
function renderScene()
  -- Clear the screen and depth buffer
  glClear(GL_DEPTH_BUFFER_BIT + GL_COLOR_BUFFER_BIT)

  -- Tell the camera/eye what to look at
  gluLookAt(0,0,6, 0,0,0, 0,1,0)
  glMatrixMode(GL_MODELVIEW)

  -- Draw the scene.
  glPushMatrix()
    glLoadMatrix(rotatemat)
    glTranslate(2,0,0)
    CallList:draw("sphere")
    glTranslate(-4,0,0)
    CallList:draw("poly")
  glPopMatrix()

  _glErrorCheck("after render")
end

-- Create our window and specify a name and window rendering function
wmain = Window:create{
  title = "Doris: Texture generation",
  render = renderScene -- specify the redraw function for this window
}

-- Put a sub window at the bottom of the screen
sw=SubWindow{ parent=wmain, side="bottom" }

-- Add an object rotator to the bottom of the screen that
-- manipulates the matrix we created.
Rotator{
    parent=sw, -- parent of this widget
    text="Rotate scene", -- text label for this widget
    value=rotatemat, -- matrix that rotator manipulates
    line="|", -- vertical spacing line drawn
    spin=0.99 -- allow the rotator to spin with little damping when released
}

--for k,v in texture.fns do
--end

Listbox{
    parent=sw,
    text="Functions:",
    items=texture.fns,
    line="|",
    update = function(f)
      setFunction(f)
    end
}

createTexture()
createObjects()
setFunction(defaultfn)

